from rcssmin import cssmin

from menagerie.Items.MinifiedItemMixin import MinifiedItemMixin
from menagerie.Items.Static.StaticItem import StaticItem


class StaticCSS(MinifiedItemMixin, StaticItem):
    extensions = ('css',)
    minify_key = 'css'
    out_extension = 'min.css'
    byte_mode = False

    def minify(self, content: str) -> str:
        return cssmin(content)



import sys
from enum import IntEnum

from menagerie.utils.console import ConsoleTextStyle, FourBitConsoleColors, color_text

__all__ = (
    'LogType',
    'Logger'
)


class LogType(IntEnum):
    """
        A type of log message

        :cvar debug: Message useful while debugging, but obnoxious everywhere else
        :cvar info: Info for the user
        :cvar warning: A possible mistake the user made
        :cvar critical: Something that prevents the site from being generated
    """

    debug = 0
    info = 1
    warning = 2
    critical = 3


class Logger:
    """
        Handles logging messages to the console

        :cvar __level: The current log level
        :cvar __log_styles: The `ConsoleTextStyle` to use for each log type
    """

    def __new__(cls, *args, **kwargs):
        raise TypeError("Logger is a static class!")

    __level: LogType = LogType.info
    __log_styles: dict[LogType, ConsoleTextStyle] = {
        LogType.debug: ConsoleTextStyle(fg_color=FourBitConsoleColors.CYAN),
        LogType.info: ConsoleTextStyle(fg_color=FourBitConsoleColors.WHITE),
        LogType.warning: ConsoleTextStyle(fg_color=FourBitConsoleColors.YELLOW),
        LogType.critical: ConsoleTextStyle(fg_color=FourBitConsoleColors.RED, bold=True)
    }

    @classmethod
    def update_level_from_string(cls, level_str: str) -> None:
        cls.update_level(['debug', 'info', 'warning', 'critical'].index(level_str.lower()))

    @classmethod
    def update_level(cls, new_level: LogType) -> None:
        cls.__level = new_level

    @classmethod
    def log(cls, message: str, log_type: LogType) -> None:
        """
            Logs a message to the console if the passed type is above the current level

            :param message: Message to print
            :type message: str
            :param log_type: The type of message to log
            :type log_type: LogType
        """

        # TODO: Log to file eventually
        if int(log_type) >= int(cls.__level):
            cls.__print(message, log_type)

    @classmethod
    def log_error(cls, message: str, should_exit: bool = True) -> None:
        cls.log(message, LogType.critical)
        if should_exit:
            sys.exit(1)

    @classmethod
    def log_warning(cls, message: str) -> None:
        cls.log(message, LogType.warning)

    @classmethod
    def log_info(cls, message: str) -> None:
        cls.log(message, LogType.info)

    @classmethod
    def log_debug(cls, message: str) -> None:
        cls.log(message, LogType.debug)

    @classmethod
    def force_log(cls, message: str, log_type: LogType) -> None:
        """
            Similar to `log()`, but doesn't check against `__level`
        """

        cls.__print(message, log_type)

    @classmethod
    def __print(cls, message: str, log_type: LogType) -> None:
        print(color_text(message, cls.__log_styles.get(log_type), reset=True))



import re

from bs4 import BeautifulSoup

from menagerie.Items.Pages.AbstractPage import AbstractPage

__all__ = ('HTMLPage',)

HEADER_PATTERN = re.compile('^h[1-6]$')


class HTMLPage(AbstractPage):
    extensions = ('jinja2', 'html', 'htm')

    def inner_render(self, content: str) -> str:
        return self.manager.env.from_string(content).render(page=self, **self.manager.context)

    def setup_toc(self, content):
        soup = BeautifulSoup(content, 'html.parser')
        toc_raw = []
        for match in soup.find_all(HEADER_PATTERN):
            if 'id' in match.attrs.keys():
                toc_raw.append({
                    'level': int(match.name[1]),
                    'id': match['id'],
                    'name': match.text,
                    'parent': None,
                    'children': []
                })
        root = []
        current_scope = root
        current_parent = None
        current_level = 1
        for index, item in enumerate(toc_raw):
            if item['level'] > current_level:
                current_parent = toc_raw[index - 1]
                current_level = item['level']
                current_scope = current_parent['children']
            elif item['level'] < current_level:
                curr_p = toc_raw[index - 1]
                while curr_p['level'] != item['level']:
                    curr_p = curr_p['parent']
                current_parent = curr_p['parent']
                current_scope = current_parent['children'] if current_parent is not None else root
                current_level = item['level']

            current_scope.append(item)
            item['parent'] = current_parent
        self.meta['table_of_contents'] = root if root is not None else None

    def load_metadata(self):
        content = self.get_content()
        for match in re.findall(r"{#~(.*?)~#}", content, re.MULTILINE):
            seperated = match.strip().split(':')
            self.meta[seperated[0].lower().replace('-', '_')] = seperated[1]
        if self.meta['render_toc']:
            self.setup_toc(content)



from abc import ABC, abstractmethod
from datetime import datetime
from pathlib import Path

from jinja2 import Environment, PackageLoader

from menagerie.Items.AbstractItem import AbstractItem
from menagerie.utils.str_util import remove_preceding_slash

__all__ = ('AbstractManager',)

from menagerie.SiteGen import SiteGen


class AbstractManager(ABC):
    item_types: tuple
    items: list[AbstractItem] = []
    root_dir: Path
    base_env: Environment = Environment(loader=PackageLoader('menagerie.base_templates', '.'))
    gen: SiteGen = None

    def __new__(cls, *args, **kwargs):
        if cls.root_dir is None:
            raise NotImplementedError("Root Dir is None")
        if cls.item_types is None:
            raise NotImplementedError("Item Types is None")
        return super(AbstractManager, cls).__new__(cls)

    def __init__(self, site_gen: SiteGen):
        self.items = []
        self.gen = site_gen
        self.base_env.globals.update({
            'settings': self.gen.settings,
            'gen_time': datetime.now().strftime("%d/%m/%Y")
        })
        self.base_env.filters.update({
            'full_url': lambda relative, add_prefix=True: self.gen.settings['base_url'] + (remove_preceding_slash(self.gen.settings['url_prefix']) if add_prefix else "") + remove_preceding_slash(relative)
        })

    def find(self):
        matches = []
        for item_type in self.item_types:
            for ext in item_type.extensions:
                for path in Path(self.gen.settings['content_dir'], self.root_dir).glob(f'**/*.{ext}'):
                    if str(path) not in matches:
                        self.items.append(item_type(self, path.relative_to(self.gen.settings['content_dir'])))
                        matches.append(str(path))

    def get_items(self):
        return self.items

    @abstractmethod
    def initialize(self):
        pass

    def generate(self):
        raise NotImplementedError()



import re
from pathlib import Path

from markdown import Markdown, Extension
from markdown.treeprocessors import Treeprocessor

from menagerie.Items.Pages.AbstractPage import AbstractPage

__all__ = ('MDPage',)

ADDITIONAL_CLASSES: dict[str, str] = {
    'img': "img-fluid rounded mx-auto d-flex",
    'table': "table table-striped"
}


class MTreeProcessor(Treeprocessor):

    def __init__(self, page):
        super().__init__()
        self.page = page

    def run(self, node):
        for child in node:
            self.page.process_node(child)
        return node


class MenagerieMarkdownExtension(Extension):

    def __init__(self, page, **kwargs):
        self.page = page
        super().__init__(**kwargs)

    def extendMarkdown(self, md: Markdown) -> None:
        md.registerExtension(self)
        self.tree = MTreeProcessor(self.page)
        self.tree.md = md
        self.tree.config = self.getConfigs()
        md.treeprocessors.add('menagerie_tree', self.tree, '_end')


class MDPage(AbstractPage):
    extensions = ('md', 'markdown')

    md: Markdown

    def process_node(self, node) -> None:
        if node.tag in ADDITIONAL_CLASSES.keys():
            node.set('class', ADDITIONAL_CLASSES[node.tag])
        if node.tag == 'img':
            possible_match = re.match(r"{{ ?[\"'](.*?)['\"]\|static.*?}}", node.get('src'))
            if possible_match:
                image_path = possible_match.group(1)
                size = self.manager.gen.shared_info['image_sizes'].get(str(Path(image_path).as_posix()), (100, 100))
                node.set('width', str(size[0]))
                node.set('height', str(size[1]))
        for child in node:
            self.process_node(child)

    def __init__(self, manager, path: Path):
        super(MDPage, self).__init__(manager, path)
        md_settings = {
            'extensions': ['extra', 'toc', 'meta', MenagerieMarkdownExtension(self)],
            'output-format': 'html5'
        }
        self.md = Markdown(**md_settings)

    def load_metadata(self) -> None:
        self.md.convert(self.get_content()).replace('&quot;', '"')
        self.meta.update({k: None if v is None else v[0] for k, v in self.md.Meta.items()})
        self.meta['table_of_contents'] = self.md.toc_tokens

    def inner_render(self, content: str) -> str:
        converted = self.md.convert(self.get_content()).replace('&quot;', '"')
        return self.manager.env.from_string(converted).render(page=self, **self.manager.context)



import sys
from pathlib import Path
from textwrap import dedent

from menagerie.generate import generate
from menagerie.new_project import new_project

__version__ = '0.1.1'
__help__ =  dedent("""
                Usage:
                    menagerie new-project [name] : Creates a new menagerie project with a given name
                    menagerie generate [config] : Generates a site with the given config
                    menagerie help : Shows this message
                    menagerie version : Shows the version
            """)


def execute_from_commandline(args: list[str] = None):
    if args is None:
        args = sys.argv
    match [arg.lower() for arg in args[1:]]:
        case ['new-project']:
            new_project("NewProject")
        case ['new-project', name]:
            new_project(name)
        case ['generate']:
            generate(Path('config.json'))
        case ['generate', Path(config_path)]:
            generate(config_path)
        case ['version']:
            print(f"Menagerie Version: {__version__}")
        case ['help']:
            print(__help__)
        case _:
            print(f"Invalid Command '{' '.join(sys.argv)}'")
            print(__help__)



from abc import ABC, abstractmethod
from pathlib import Path

from htmlmin import minify as html_minify
from jinja2.environment import Markup

from menagerie.Items.AbstractItem import AbstractItem
from menagerie.Items.MinifiedItemMixin import MinifiedItemMixin

__all__ = ('AbstractPage', 'MINIFY_SETTINGS')

from menagerie.utils.str_util import pretty_title

MINIFY_SETTINGS = {
    'remove_empty_space': True,
    'keep_pre': True,
    'remove_optional_attribute_quotes': False
}


class AbstractPage(MinifiedItemMixin, AbstractItem, ABC):
    base_template = "page_base.jinja2"
    byte_mode = False
    out_extension = 'html'
    minify_key = 'html'

    def __init__(self, manager, path: Path):
        super().__init__(manager, path)
        self.meta: dict[str, object] = {
            'title': None,
            'description': self.manager.gen.settings['brand']['meta']['description'],
            'sort_priority': 10,
            'hide_in_nav': False,
            'out_file': None,
            'render_toc': self.manager.gen.settings['default-toc'],
            'table_of_contents': None
        }

    @abstractmethod
    def load_metadata(self) -> None:
        pass

    @abstractmethod
    def inner_render(self, content: str) -> str:
        pass

    def initialize(self) -> None:
        self.load_metadata()
        if self.meta['out_file'] is not None:
            self.out_path = self.out_path.with_stem(self.meta['out_file'])
        if self.meta['title'] is None:
            self.meta['title'] = pretty_title(self.in_path.stem)
        if str(self.meta['render_toc']).lower() == 'false' or self.meta['table_of_contents'] is None:
            self.meta['render_toc'] = False
        self.meta['hide_in_nav'] = str(self.meta['hide_in_nav']).lower() != 'false'
        self.meta['sort_priority'] = int(self.meta['sort_priority'])

    def transform(self, content: str) -> str:
        base_template = self.manager.base_env.get_template(self.base_template)
        return base_template.render(page=self, content=Markup(self.inner_render(content)), **self.manager.context)

    def minify(self, content: str) -> str:
        return html_minify(content, **MINIFY_SETTINGS)



from json import JSONDecoder, JSONDecodeError

from json_schema_for_humans.generate import generate_schemas_doc
from json_schema_for_humans.generation_configuration import GenerationConfiguration
from json_schema_for_humans.schema.schema_to_render import SchemaToRender
from json_schema_for_humans.template_renderer import TemplateRenderer

from menagerie.Items.Pages.AbstractPage import AbstractPage
from menagerie.utils.logger import Logger
from menagerie.utils.no_print import NoPrint

SCHEMA_SETTINGS = GenerationConfiguration()
SCHEMA_SETTINGS.link_to_reused_ref = False
SCHEMA_SETTINGS.minify = False


class JSONSchema(AbstractPage):
    base_template = 'schema_templates/schema_template.jinja2'
    extensions = ('json', 'jsonc')

    def load_metadata(self) -> None:
        try:
            decoded = JSONDecoder().decode(self.get_content())
            self.meta.update({k.strip().lower(): v for k, v in decoded.get("$docs", {}).items()})
        except JSONDecodeError:
            Logger.log_error(f"Couldn't parse json file: {self.in_path.name}")

    def inner_render(self, content: str) -> str:
        schema_renderer = TemplateRenderer(SCHEMA_SETTINGS)
        self.manager.base_env.filters.update(schema_renderer.template.environment.filters)
        self.manager.base_env.tests.update(schema_renderer.template.environment.tests)
        self.manager.base_env.globals.update(schema_renderer.template.environment.globals)
        schemas = [SchemaToRender(self.get_path_to_open(), None, None)]
        schema_template = self.manager.base_env.get_template("schema_templates/json/schema_base.jinja2")
        template_renderer = TemplateRenderer(SCHEMA_SETTINGS, schema_template)
        template_renderer.render = lambda inter: self.template_override(template_renderer, inter,
                                                                        **self.manager.context)
        with NoPrint():
            rendered = generate_schemas_doc(schemas, template_renderer)
        return rendered[str(self.in_path.name)]

    def template_override(self, template: TemplateRenderer, intermediate_schema, **context):
        template.template.environment.loader = self.manager.base_env.loader
        rendered = template.template.render(schema=intermediate_schema, config=SCHEMA_SETTINGS,
                                            title=self.meta['title'], **context)
        return rendered
